# -*- coding: ISO-8859-1 -*-
"""CapellaScript -- R. v. Salis, Ergnzung um die Bassseite Peter Becker 
>>> Swiss_Oergeli : Tabulaturschrift fr das Schweizer Oergeli

    Setzt Noten in die Tabulaturschrift fr Schwyzerrgeli um.     
<<<
History: 23.06.2005 Version 1.0.0 RVS - erste Ausgabe
         12.01.2015 Version 1.0.1 PB  - Absturz verhindert
         20.01.2015 Version 2.0.0 PB  - Untersttzung der Basstne
"""
import xml.dom, tempfile
from caplib.capDOM import ScoreChange
from xml.dom.minidom import NodeList

version = '2.0.0'

verboten      = (51,52,59,76,83,89,90)                                         # nicht existierende Tne aus der chromatischen Leiter
ziehen        = (0,0,0,2,0,0,56,4,3,0,6,52,5,8,1,7,58,10,9,12,62,54,93,60,13,16,0,15,64,18,17,20,66,0,19,68,21,70,0,0,0,0)
stossen       = (2,0,0,4,52,3,0,0,90,0,0,56,8,7,1,10,0,9,60,0,91,54,58,62,14,13,0,16,0,15,64,0,92,0,66,68,20,19,70,0,0,21)
notenListe    = []
griffListe    = []    
druckzugListe = []                                                             # Fr jeden Takt die Richtung +1 oder -1
taktListe     = [1]                                                            # Notennummer der ersten Note in jedem Takt
barListe      = [0]                                                            # Taktnummer des ersten Taktes in jedem Abschnitt
ausnahmeListe = []                                                             # Noten im Musikstck, die auf dem Oergeli nicht gespielt werden knnen
verdecktListe = []                                                             # verdeckte Kreuztne
merkTime      = 0
xlateB        = dict(G='1',C='2',F='3',Bb='4',Eb='5',Ab='6',Db='7',Gb='8',B='9')  # Bsse bersetzen


def getKoordinaten(chromatic):
    #Wandelt Tonhhe in Koordinaten um
    
    if chromatic < 50 or chromatic > 91 or chromatic in verboten:
        clef=[-1,-1]
    else:
        clef=[ziehen[chromatic-50],stossen[chromatic-50]]
    return clef
            
def analyseScore(score): 
    global takt
    global merkTime
    for s in score.systems():   
        takt = float(s.staff(0).defaultMeter())                                # Ermittelt Takt
    m=score.nSystems()
    n = 0                                                                      # Notennummer
    newTime = 0
    Flag = True
    offsetTime = 0
    for k in range(0,m,1): 
        for a in score.system(k).noteObjs():
            if a.isBarline():
                if (k == 0 and len(taktListe) == 1):                           # Auftakt ist vorhanden
                    Flag = False
                if (len(taktListe) > 1 and barListe[len(barListe)-1] != len(taktListe)) :
                    barListe.append(len(taktListe))                            # Stellung des festen Taktstriches in barListe
            if a.isChord():
                n +=1                                                          # Notenzhler erhhen
                oldTime = newTime
                t = a.time()
                tonh = score.system(k).pitches(t,True,False)                   # Holt Notenwerte des Akkords
                j=0
                liste=[]
                for b in a.heads():
                    ton=tonh[j];
                    listeneintrag = getKoordinaten(ton)                        # Wandelt sie in Koordinaten um
                    liste.append(listeneintrag)
                    j += 1
                notenListe.append(liste)                                       # Ergnzt notenListe mit Akkord
                if Flag == False:                                              # Auftakt ist vorhanden
                    offsetTime = t
                    merkTime = t
                    Flag = True
                newTime = int((t-offsetTime)/takt)
                if newTime-oldTime != 0:
                    taktListe.append(n)                                        # Stellung des Taktstriches in taktListe
            elif a.isRest():                                                   # nur wirksam, falls Auftakt vorhanden und ->
                if Flag == False:                                              # erstes Objekt nach dem ersten Taktstrich ->
                    t = a.time()                                               # eine Pause ist
                    offsetTime = t
                    Flag = True            
        offsetTime = 0
    taktListe.append(n+1)                                                      # Taktlistenabschluss
    L = len(taktListe) - 1
    if barListe[len(barListe)-1] != L:                                         # Barlistenabschluss
        barListe.append(L)

def ausgleichen(abschnitt):                                                    # erzeugt pro Abschnitt eine gerade Anzahl von ->
    n = len(abschnitt)                                                         # etwa gleich langen Unterabschnitten
    if n == 0:
        return
    m = n/10
    if n%10 <> 0:
        m += 1
    m = 2 * m                                                                  # Anzahl Hinundher pro Abschnitt
    hh = n / m                                                                 # Anzahl Takte pro Hinundher
    hhrest = n % m                                                             # Resttakte
    info = []
    eintrag = 1
    if n == 1:
        info.append(eintrag)
    else:
        for k1 in range(0,m,1):
            for k2 in range(0,hh,1):
                info.append(eintrag)
                if hhrest > 0:
                    info.append(eintrag)
                hhrest -= 1
            eintrag = -eintrag
    for k in range(0,n,1):
        test = info[k]+abschnitt[k]
        if test > 2:
            messageBox('Achtung','Takt muss aufgespalten werden, andere Stimmung probieren')
        elif test == 0:
            info[k]=-info[k]                
    test = 0
    for k in range(0,n,1):                                                     # Summe bilden
        test += info[k] 
    if test > 0:                                                               # zu viele Zugtakte pro Abschnitt
        for k in range(1,n,1):
            if (abschnitt[k] == 0 and info[k] == 1):
                info[k] = -1
                test -= 1
                if test < 2:
                    break
    elif test < 0:                                                             # zu viele Drucktakte pro Abschnitt
        for k in range(1,n,1):
            if (abschnitt[k] == 0 and info[k] == -1):
                info[k] = 1
                test += 1
                if test > 0:
                    break
    return info

def createRichtung():                                                          # fr jeden Takt wird die mgliche Balgrichtung ermittelt                      
    global druckzugListe
    richtungsListe = []
    m_alt = 1
    for m in taktListe:
        if m > 1:
            druck = 1
            zug = 1
            for k in range(m_alt,m,1):
                akkord = notenListe[k-1]
                for i in akkord:
                    clef = i
                    if clef[1] == 0:
                        zug = 0
                    if clef[0] == 0:
                        druck = 0
            if (zug == 0 and druck == 0):
                richtungsListe.append(4)                                       # Takt muss aufgespalten werden
            if (zug == 0 and druck == 1):
                richtungsListe.append(1)                                       # Takt muss auf Druck gespielt werden
            if (druck == 0 and zug == 1):
                richtungsListe.append(-1)                                      # Takt muss auf Zug gespielt werden
            if (druck == 1 and zug == 1):
                richtungsListe.append(0)                                       # Takt kann in beiden Richtungen gespielt werden
        m_alt = m
    for k in range(0,len(barListe)-1,1):
        hinzu = richtungsListe[barListe[k]:barListe[k+1]]
        druckzugListe.append(hinzu)
    richtungsListe=[]
    for a in druckzugListe:                                                    # abschnittsweise ausgleichen
        richtungsListe.append(ausgleichen(a))
    druckzugListe = []
    for a in richtungsListe:
        for b in a:
            druckzugListe.append(b)
    return druckzugListe                                                       # zu jedem Takt die Richtung liefern

def berechneGriff(note):
    info1 = ['D5','B5','A6','E6']                                              # kann dem Spieler angepasst werden 
    info2 = ['E4','F4','G4','A4','B4','C5','D5','E5','F5','G5','A5','B5',
             'C6','D6','E6','F6','G6','A6','B6','C7','D7','E7',]
    if note > 70:
        griff = info1[note-90]
    elif note > 21:
        griff = info2[note-50]+'x'
    elif note >= 0:
        griff = info2[note]
    else:
        griff = 'XX'
    return griff

def createGriff(druckzugListe):
    n = len(taktListe)
    for k in range(0,n-1,1):
        beginn = taktListe[k]
        ende = taktListe[k+1]
        richtung = druckzugListe[k]
        for j in range(beginn,ende,1):
            notenInfo = notenListe[j-1]
            remember = []
            for a in notenInfo:                                                # es knnte ein Akkord sein
                if richtung == 1:                                              # stossen
                    note = a[0]
                else:                                                          # ziehen
                    note = a[1]
                info = berechneGriff(note)
                for r in remember:
                    if info[:2] == r:
                        info += '*'                                            # verdeckte Kreuznote
                        verdecktListe.append(j)
                remember.append(info[:2])
                if info == 'XX':                                               # Griff existiert nicht
                    ausnahmeListe.append(j-1)
                griffListe.append(info)       

def faerbeNoten(druckzugListe):
    if len(ausnahmeListe) == 0:
        return
    else:
        messageBox('Alert','Einzelne Noten knnen nicht gegriffen werden. Die Akkorde, in denen sie vorkommen, werden rot eingefrbt')
        for j in ausnahmeListe:                                                # Nicht spielbare Akkorde rot einfrben
            k = 0
            for a in activeScore().noteObjs():
                if a.isChord():
                    if k == j:
                        a.setColor(Color.RGB(255,0,0))
                    k += 1
       
def drawBalken(breite,dicke):
    return dict(
        type = 'line',
        x1 = -2, y1 = 9,
        x2 = -2 + breite, y2 = 9,
        lineWidth = dicke)

def druckBalken(score):                                                        # Kodiert Druck- (dick) und Zugrichtung (dnn) ->
    m=score.nSystems()                                                         # als Linie unter den Notenzeilen
    tak = 1                                                                    # Taktnummer
    n = 0
    for a in score.system(0).noteObjs():                                       # Erste Zeile des Musikstcks
        if (a.isChord() and n == 0):                                           # Erste Note
            b = a
            x0 = a.posX(True)
        if (a.isChord() and n < taktListe[1]-1):                               # sucht Ende des ersten Taktes
            n += 1
        elif (a.isChord() or a.isRest()):       
            t = a.time()
            test = (t-merkTime)/takt
            if test-tak >=0:                                                   # neuer Takt beginnt
                x1 = a.posX(True)
                breite = (x1 - x0)
                dicke = 0.4 - 0.2 * druckzugListe[tak-1]
                b.addDrawObj(drawBalken(breite,dicke))
                b = a                                                          # merke erste Note oder Pause im Takt
                x0 = x1
                tak += 1
    x1 = a.posX(True) + 7                                                      # letzter Takt in der ersten Zeile
    if a.isBarline():
        x1=x1-5
    breite = (x1 - x0)
    dicke = 0.4 - 0.2 * druckzugListe[tak-1]
    b.addDrawObj(drawBalken(breite,dicke))
    for k in range(1,m,1):                                                     # alle anderen Zeilen
        n = 0
        ta = 1
        for a in score.system(k).noteObjs():
            if ((a.isChord() or a.isRest()) and n == 0):                       # erste Note oder Pause in der Zeile
                b = a
                x0 = a.posX(True)
                n += 1
            elif (a.isChord() or a.isRest()):
                t = a.time()
                test = t/takt
                if  test-ta >= 0:                                              # neuer Takt beginnt
                    ta += 1
                    tak += 1
                    x1 = a.posX(True)
                    breite = (x1 - x0)
                    dicke = 0.4 - 0.2 * druckzugListe[tak-1]
                    b.addDrawObj(drawBalken(breite,dicke))
                    b = a                                                      # merke erste Note oder Pause im Takt
                    x0 = x1
        x1 = a.posX(False) + 7
        if a.isBarline():
            x1=x1-5
        breite = (x1 - x0)
        if tak < len(druckzugListe):
            dicke = 0.4 - 0.2 * druckzugListe[tak]
            b.addDrawObj(drawBalken(breite,dicke))
        tak += 1


def Text():
    return dict(
        type = 'text',
        x = -3,
        y = -6,
        content = 'Schwyzerrgeli B')

def writeSchwyzeroergeli(score):
    k = 0
    for obj in score.noteObjs():
        if k > 0:
            return
        if obj.isChord():
            obj.addDrawObj(Text())
            k += 1
    
def getElementObjects(objList):   
    newList = NodeList()
    for n in range(objList.length):
        if objList[n].nodeType == objList[n].ELEMENT_NODE:
            newList.append(objList[n])
    return newList

def addElementNode(el,tagName):
    # Neue Node zu "el" hinzufgen wenn Node "tagName" nicht existiert
    # ansonsten existierende Node zurckmelden
    global doc
    childs = el.childNodes
    for n in range(childs.length):
        if childs[n].nodeType ==childs[n].ELEMENT_NODE and childs[n].tagName == tagName:
            return childs[n]
    newChild = doc.createElement(tagName)
    el.appendChild(newChild)
    return newChild

def addNewElementNode(el,tagName):
    # add new Node with tagName "tagName" to el 
    global doc
    newChild = doc.createElement(tagName)
    el.appendChild(newChild)
    return newChild

def insertHead(el,wert):
    # unsichtbaren Kopf einfgen    
    chord    = addNewElementNode(el,'chord')
    duration = addNewElementNode(chord,'duration')
    display  = addNewElementNode(chord,'display')
    heads    = addNewElementNode(chord,'heads')
    head     = addNewElementNode(heads,'head')
    duration.setAttribute('base',wert)
    head.setAttribute('pitch','B5')
    display.setAttribute('invisible','true')


def insertText(score,el,wert,groesse,farbe):
    # Text einfgen

    drawObjs     = addElementNode(el,'drawObjects')
    newdrawObj1  = addNewElementNode(drawObjs,'drawObj')
    newText1     = addNewElementNode(newdrawObj1,'text')
    newFont1     = addNewElementNode(newText1,'font')
    newContent1  = addNewElementNode(newText1,'content')
    newText1.setAttribute('x','0')
    newText1.setAttribute('y','1.165')
    newFont1.setAttribute('face','Times New Roman')
    newFont1.setAttribute('height',str(groesse))
    newFont1.setAttribute('weight',"700")
    newFont1.setAttribute('charSet',"1")
    if farbe == 1:
        newFont1.setAttribute('color',"FF0000")
    newContent1.appendChild(score.parentNode.createTextNode(str(wert)))


def updateTaktinhalt(child, taktinhalt):
    #t aktinhalt um Wert der Note hochzhlen
    
    durationD = child.getElementsByTagName('duration')                           # den Taktinhalt hochzhlen
    base = durationD[0].getAttribute('base')
    if durationD[0].hasAttribute('dots'):
        dots = durationD[0].getAttribute('dots')
    else:
        dots = 0

    if base == '1/16':
        taktinhalt = taktinhalt + 0.0625
        if dots == '1':
            taktinhalt = taktinhalt + 0.03125
    elif base == '1/8':
        taktinhalt = taktinhalt + 0.125
        if dots == '1':
            taktinhalt = taktinhalt + 0.0625
    elif base == '1/4':
        taktinhalt = taktinhalt + 0.25
        if dots == '1':
            taktinhalt = taktinhalt + 0.125
    elif base == '1/2':
        taktinhalt = taktinhalt + 0.5
        if dots == '1':
            taktinhalt = taktinhalt + 0.25
    else:
        if takt == '3/4' or takt == '6/8':
                    taktinhalt = taktinhalt + 0.75
        else:
            taktinhalt = taktinhalt + 1
        if dots == '1':
            taktinhalt = taktinhalt + 0.5
    return taktinhalt

def insertPlaceholder(childsD):
    # Platzhalter fr Bassbegleitung einfgen
    global bass, nOs, time, taktTs, nenner, teiler, headTyp

    for ch in range(childsD.length):                                           # komplette Zeile bearbeiten
        #messageBox('child',str(childsD[ch].tagName))
        if childsD[ch].tagName == 'timeSign':                                  # Taktzahl einlesen 
            timeSign = childsD[ch].getAttribute('time')
            if timeSign == 'C':
                taktTs = '4/4'
            elif timeSign == 'allaBreve':
                taktTs  = '2/2'
            elif timeSign == 'longAllaBreve':
                taktTs = '4/2'
            else:
                taktTs = timeSign
            TS = addNewElementNode(nOs[0],'timeSign')                          # und wegen korrekter Formatierung unsichtbar ausgeben
            TS.setAttribute('time',taktTs)
            TS.setAttribute('color','FFFFFF')
            #messageBox('TS',str(taktTs))

            taktTeilung = taktTs.split("/",2)                                  # in Zhler und Nenner aufspalten
            zaehler = float(taktTeilung[0])
            nenner  = float(taktTeilung[1])
            time = zaehler / nenner                                            # und als floating Number merken
            taktinhalt = 0                                                     # Taktinhalt auf 0 setzen
            
            if bass == 1:                                                      # abhngig von den gewhlten Optionen und dem Takt
                teiler = 0.25                                                  # den entsprechenden Head Typ und Platzhaltertyp setzen
                headTyp = '1/4'
                if nenner == 8:
                    teiler = 0.125
                    headTyp = '1/8'
            elif bass == 2:
                teiler = 0.125
                headTyp = '1/8'
                    
        elif childsD[ch].tagName == 'barline' and time <> 0:                   # potentiellen Auftakt bearbeiten

            if bass == 1:                                                      # abhngig von den gewhlten Optionen und dem Takt
                teiler = 0.25                                                  # den entsprechenden Head Typ und Platzhaltertyp setzen
                headTyp = '1/4'
                if nenner == 8:
                    teiler = 0.125
                    headTyp = '1/8'
            elif bass == 2:
                teiler = 0.125
                headTyp = '1/8'
                
            count = taktinhalt / teiler
            countInt = int(count)
            delta = taktinhalt - (countInt * teiler)

            ix = 0
            if delta <> 0:                                                     # ungeraden Wert ausgeben
                if delta == 0.0625:
                    wert = '1/16'
                elif delta == 0.125:
                    wert = '1/8'
                elif delta == 0.25:
                    wert = '1/4'
                elif delta == 0.5:
                    wert = '1/2'
                insertHead(nOs[0],wert)
                
            while ix <> countInt:                                              # restliche geraden Werte ausgeben
                insertHead(nOs[0],headTyp)
                ix = ix +1
                
            addNewElementNode(nOs[0],'barline')                                # festen Taktstrich zur sauberen Formatierung ausgeben
            taktinhalt = 0
            
        elif childsD[ch].tagName == 'clefSign':                                # Falls Schlssel Taktinhalt auf 0 setzen
            taktinhalt = 0
            
        elif childsD[ch].tagName == 'chord' or childsD[ch].tagName == 'rest' and time <> 0:    # Bei Pausen oder Noten
            taktinhalt = updateTaktinhalt(childsD[ch], taktinhalt)             # den Taktinhalt hochzhlen

        if taktinhalt == time and time <> 0:                                   # und wenn der Taktinhalt einem vollen Takt entspricht
            #messageBox('TI=T',str(taktinhalt) + '\n' + str(time))
            count = taktinhalt / teiler
            ix = 0
            while ix <> count:
                insertHead(nOs[0],headTyp)                                     # diesen ausgeben
                ix = ix +1
            taktinhalt = 0

    if taktinhalt <> 0:                                                        # falls am Ende der Zeile ein unvollstndiger Takt existiert
        count = taktinhalt / teiler
        countInt = int(count)
        delta = taktinhalt - (countInt * teiler)

        ix = 0
        while ix <> countInt:                                                  # geraden Werte ausgeben
            insertHead(nOs[0],headTyp)
            ix = ix +1
        if delta <> 0:                                                         # restlichen ungeraden Wert ausgeben
            if delta == 0.0625:
                wert = '1/16'
            elif delta == 0.125:
                wert = '1/8'
            elif delta == 0.25:
                wert = '1/4'
            elif delta == 0.5:
                wert = '1/2'
            insertHead(nOs[0],wert)

        taktinhalt = 0                                                         # und Taktinhalt sicherheitshalber wieder auf 0 setzen

def moveChords(childsD, childsB, voice):
    # Begleitakkorde von der Diskant- zur Bassseite verschieben

    taktinhaltD = 0
    taktinhaltB = 0
    for ch in range(childsD.length):                                           # komplette Zeile bearbeiten
        if childsD[ch].tagName == 'chord' or childsD[ch].tagName == 'rest':    # Bei Pausen oder Noten
            taltinhaltDsave = taktinhaltD
            taktinhaltD = updateTaktinhalt(childsD[ch], taktinhaltD)           # den Taktinhalt Diskant hochzhlen
            DOs = childsD[ch].getElementsByTagName('drawObjects')
            if DOs:
                transposable = DOs[0].getElementsByTagName('transposable')     # und transponierbare Objekte suchen
                if transposable:
                    tr = transposable[0].cloneNode(1)
                    taktinhaltB = 0
                    ix = 0
                    while taktinhaltB < taltinhaltDsave:                       # wenn gefunden, dann in die Basslinie transferieren
                        if childsB[ix].tagName == 'chord' or childsB[ix].tagName == 'rest':
                            taktinhaltB = updateTaktinhalt(childsB[ix], taktinhaltB)
                        ix = ix + 1
                    if childsB[ix].tagName == 'chord' or childsB[ix].tagName == 'rest':  
                        DOsB = addElementNode(childsB[ix],'drawObjects')
                    else:
                        iix = ix
                        while childsB[iix].tagName <> 'chord' or childsB[iix].tagName == 'rest':
                            iix = iix+1
                            DOsB = addElementNode(childsB[iix],'drawObjects')
                    DOB  = addNewElementNode(DOsB,'drawObj')
                    DOB.appendChild(tr)
           
    deleteSimpleObject(voice,'transposable')                                   # und die Akkorde in der Diskantzeile lschen
        
    return

def xlateChords(score,childsB, voice):
    # Begleitakkorde bersetzen und Rhythmus erzeugen

    global rhyt, bass, anfang, start, anzahlPlatzhalter, muster

    anfang = 0
    basston = 0
    ix = 0
    for ch in range(childsB.length):                                           # komplette Zeile bearbeiten
        if childsB[ch].tagName == 'timeSign':                                  # Taktzahl einlesen 
            timeSign = childsB[ch].getAttribute('time')
            start = 0
            taktTeilung = timeSign.split("/",2)                                # in Zhler und Nenner aufspalten
            zaehler = float(taktTeilung[0])
            nenner  = float(taktTeilung[1])
            
            if (zaehler == 6 or zaehler == 3) and rhyt == 0:                   # Rhythmuspattern vorgeben
                anzahlPlatzhalter = zaehler * 2
                muster = (1,0,0,0,0,0)
            elif (zaehler == 6 or zaehler == 3) and rhyt == 1:
                anzahlPlatzhalter = zaehler
                muster = (1,0,0,1,0,0)
            elif (zaehler == 6 or zaehler == 3) and rhyt == 2:
                anzahlPlatzhalter = zaehler 
                muster = (1,1,1,1,1,1)
            elif bass == 2 and nenner == 8 and rhyt == 1:
                anzahlPlatzhalter = zaehler
                muster = (1,0,1,0,1,0,1,0)
            elif bass == 2 and nenner == 4 and rhyt == 0:
                anzahlPlatzhalter = zaehler * 2
                muster = (1,0,0,0,0,0,0,0)
            elif bass == 2 and nenner == 4 and rhyt == 2:
                anzahlPlatzhalter = zaehler * 2
                muster = (1,1,1,1,1,1,1,1)
            elif bass == 2 and nenner == 4 and rhyt == 1:
                anzahlPlatzhalter = zaehler * 2
                muster = (1,0,1,0,1,0,1,0)
            elif bass == 1 and nenner == 8 and rhyt == 0:
                anzahlPlatzhalter = zaehler
                muster = (1,0,0,0,0,0,0,0)
            elif bass == 1 and nenner == 8 and rhyt == 2:
                anzahlPlatzhalter = zaehler
                muster = (1,1,1,1,1,1,1,1)
            elif bass == 1 and nenner == 8 and rhyt == 1:
                anzahlPlatzhalter = zaehler
                muster = (1,0,1,0,1,0,1,0)
            elif bass == 1 and nenner == 4 and rhyt == 0:
                anzahlPlatzhalter = zaehler
                muster = (1,0,0,0,0,0,0,0)
            elif bass == 1 and nenner == 4 and rhyt == 2:
                anzahlPlatzhalter = zaehler
                muster = (1,1,1,1,1,1,1,1)
            elif bass == 1 and nenner == 4 and rhyt == 1:
                anzahlPlatzhalter = zaehler
                muster = (1,0,1,0,1,0,1,0)

            start = 0
            ix = 0
            
        elif childsB[ch].tagName == 'chord' or childsB[ch].tagName == 'rest':  # Bei Pausen oder Noten

            DOs = childsB[ch].getElementsByTagName('drawObjects')
            if DOs:
                transposable = DOs[0].getElementsByTagName('transposable')     # nach transponierbaren Objekte suchen
                if transposable:
                    base = transposable[0].getAttribute('base')
                    if xlateB.has_key(base):
                        basston = xlateB[base]
                        insertText(score,childsB[ch],basston,'11',0)           # und als groe Zahl ausgeben
                    else:
                        basston = base
                        insertText(score,childsB[ch],basston,'11',1)           # wenn ungltiger Akkord, diesen gro rot ausgeben
                    start = 1
                    ix = ix + 1
                else:
                    if start == 1:                                             # wenn kein Auftakt vorliegt und kein transponierbarer Akkord definiert ist, 
                        ph = muster[ix]                                        # aber sonstige drawObjects definiert sind
                        if ph == 1:
                            size = '11'                                        # Zahl abhngig von der im Muster definierten Gre ausgeben
                        else:
                            size = '7'
                        insertText(score,childsB[ch],basston,size,0)
                    ix = ix + 1
            else:
                if start == 1:                                                 # wenn kein Auftakt vorliegt und kein transponierbarer Akkord definiert ist, 
                    ph = muster[ix]                                            # und auch keine sonstigen drawObjects definiert sind
                    if ph == 1:                                                # Zahl abhngig von der im Muster definierten Gre ausgeben
                        size = '11'
                    else:
                        size = '7'
                    insertText(score,childsB[ch],basston,size,0)
                ix = ix + 1
            if ix == anzahlPlatzhalter:
                ix = 0

        elif childsB[ch].tagName == 'barline':                                 # bei einem festen Taktstrich, alles auf Anfang setzen
            start = 0
            ix = 0
            
    deleteSimpleObject(voice,'transposable')                                   # und zum Schluss die Akkorde in der Basszeile lschen        

def deleteSimpleObject(voice, simpleObject):
    # zB.: drawObj transposable lschen
    
    for drawObjects in voice.getElementsByTagName('drawObjects'):
        drwObjs = getElementObjects(drawObjects.childNodes)
        for drawObj in drwObjs:
            if drawObj.tagName == 'drawObj':
                for obj in getElementObjects(drawObj.childNodes):
                    if obj.tagName == simpleObject:
                        drawObj.parentNode.removeChild(drawObj)
                        continue


def rewrite(score):
    global bass, doc, nOs, time

    doc    = score.parentNode
    time = 0
    systems = score.getElementsByTagName('system')
    if bass <> 0:                                                              # Basszeile im staffLayout eintragen
        staffLayout = score.getElementsByTagName('staffLayout')
        sL = staffLayout[0].cloneNode(1)
        staffLayout[0].parentNode.appendChild(sL)
        staffLayout = score.getElementsByTagName('staffLayout')
        staffLayout[0].setAttribute('description','Diskant')                   # korrekt bezeichen
        staffLayout[1].setAttribute('description','Bass')
        distances0 = staffLayout[0].getElementsByTagName('distances')
        distances1 = staffLayout[1].getElementsByTagName('distances')
        distances0[0].setAttribute('bottom','7')
        distances0[0].setAttribute('top','4')
        distances1[0].setAttribute('bottom','5')
        distances1[0].setAttribute('top','0')
        notation = staffLayout[1].getElementsByTagName('notation')
        notation[0].setAttribute('defaultClef','N3')                           # Basszeile als ohne Schlssel
        notation[0].setAttribute('noteLines','________')                       # und ohne Linien markieren
        layoutP = score.getElementsByTagName('layout')
        systemsP = layoutP[0].getElementsByTagName('systems')                  # System Layout anpassen
        systemsP[0].setAttribute('top','10')
        systemsP[0].setAttribute('between','2')
        systemsP[0].setAttribute('pageJustified','exceptLast') 
    k=0
    
    for sy in range(systems.length):                                            
        staves = systems[sy].getElementsByTagName('staff')
        for st in range(staves.length):
            
            # Basszeile in den Systemen als Kopie der Diskantseite einfgen
            if bass <> 0:                                                      
                node = staves[st].cloneNode(1)
                staves[st].parentNode.appendChild(node)
                voices = staves[st].getElementsByTagName('voice')
                staffs = staves[st].parentNode.getElementsByTagName('staff')
                nOs = staffs[1].getElementsByTagName('noteObjects')
                childsB = getElementObjects(staffs[1].getElementsByTagName('noteObjects')[0].childNodes)
                childsD = getElementObjects(staffs[1].getElementsByTagName('noteObjects')[0].childNodes)
                for ch in range(childsB.length):                               # und die vorhandenen Noten daraus lschen
                    childsB[ch].parentNode.removeChild(childsB[ch])
                insertPlaceholder(childsD)                                     # Platzhalter fr die gesamte Zeile einfgen
                childsB = getElementObjects(staffs[1].getElementsByTagName('noteObjects')[0].childNodes)
                moveChords(childsD, childsB, voices[0])                        # Begleitakkorde von der Diskant- zur Bassseite verschieben
                childsB = getElementObjects(staffs[1].getElementsByTagName('noteObjects')[0].childNodes)
                voice = staffs[1].getElementsByTagName('voice')
                xlateChords(score,childsB, voice[0])                           # Begleitakkorde bersetzen und Rhythmus erzeugen
                
            # Diskantzeile bersetzen
            for keySign in staves[st].getElementsByTagName('keySign'):         # Tonart entfernen
                keySign.parentNode.removeChild(keySign)                        
            for clefSign in staves[st].getElementsByTagName('clefSign'):       # Schlssel entfernen
                clefSign.parentNode.removeChild(clefSign)  
            voices = staves[st].getElementsByTagName('voice')
            for vo in range(voices.length):
                noteObjects = getElementObjects(voices[vo].getElementsByTagName('noteObjects')[0].childNodes)
                for ob in range(noteObjects.length):            
                    nv = noteObjects[ob].nodeName                 
                    if nv == 'chord':
                        heads = getElementObjects(noteObjects[ob].getElementsByTagName('heads')[0].childNodes)
                        for hd in range(heads.length):
                            griff = griffListe[k]
                            if griff[len(griff)-1]=='x':
                                heads[hd].setAttribute('shape','crossCircle')  # Kreuzton 
                            heads[hd].setAttribute('pitch',griff[:2])          # Griffbild zeichnen
                            alter = heads[hd].getElementsByTagName('alter')
                            if alter:
                                alter[0].setAttribute('step','0')              # Vorzeichen entfernen
                            if griff[len(griff)-1]=='*' and alter:             # verdeckte Kreuznote  12.1.15 Peter Becker Absturz behoben falls kein alter Tag vorliegt
                                alter[0].setAttribute('step','2')
                            k += 1

def pqDialog(score):
    # Dialog fr die Bassbegleitung
    
    global bass, rhyt

    bass  = '0'
    ryth  = '0'
    
    options = ScriptOptions() 
    opt = options.get()

    radioBass = Radio(['ohne Bsse','mit Bssen - Platzhalter Viertelnoten ausser bei n/8 Takt','mit Bssen - Platzhalter Achtelnoten durchgehend'], value=int(opt.get('bass',0)))
    radioRhyt = Radio(['3er-Takt: Aaa, 4er-Takt: Aaaa, 6er-Takt: Aaaaaa','3er-Takt: Aaa, 4er-Takt: AaAa, 6er-Takt: AaaAaa','3er-Takt: AAA, 4er-Takt: AAAA, 6er-Takt: AAAAAA'], value=int(opt.get('rhyt',0)))
    HboxBass  = HBox([radioBass],text='Bass Darstellung')
    HboxRhyt  = HBox([radioRhyt],text='Rhythmus')
    box  = VBox([HboxBass,HboxRhyt])
    
    dlg = Dialog('Swiss Oergeli  Version : ' + str(version), box)

    if dlg.run():
        bass = radioBass.value()
        rhyt = radioRhyt.value()
        opt.update(dict(bass=bass,rhyt=rhyt))
        options.set(opt)
        return True
    else:
        return False

class ScoreChange(ScoreChange):
    def changeScore(self,score):
        if pqDialog(score):
            rewrite(score)
        

#-------------------------------------------------------------------------------------------------------------
#                                         M a i n
#-------------------------------------------------------------------------------------------------------------

s = activeScore()
s.registerUndo("swiss_oergeli")
analyseScore(s)                                                                # Musikstck analysieren
liste = createRichtung()                                                       # Taktweise Balgrichtung ermitteln
createGriff(liste)
faerbeNoten(liste)                                                             # Nicht spielbare Akkorde rot einfrben
if len(ausnahmeListe) == 0:
    tempInput = tempfile.mktemp('.capx')
    tempOutput = tempfile.mktemp('.capx')
    activeScore().write(tempInput)
    
    ScoreChange(tempInput,tempOutput)
    
    activeScore().read(tempOutput)
    os.remove(tempInput)
    os.remove(tempOutput)
    druckBalken(s)
    writeSchwyzeroergeli(s)


